/* 
   Unix SMB/Netbios implementation.
   Version 1.9.
   Groupname handling
   Copyright (C) Jeremy Allison               1998-2000.
   Copyright (C) Luke Kenneth Casson Leighton 1996-2000.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* 
 * algorithmic implementation of a SURS (sid to uid resolution) table.
 * only does the local SAM, does NOT even do domain membership.
 * repeat: this is for the LOCAL SAM only, not even the BUILTIN domain.
 */

#include "includes.h"
#include "libsamba.h"
#include "sids.h"
#include "others.h"


/******************************************************************
 converts SID to a UNIX id + type.  the Domain SID is,
 and can only be, our own SID.
 ********************************************************************/
BOOL surs_algdomonly_sam_sid_to_unixid(const DOM_SID *sid, SURS_POSIX_ID *id,
				BOOL create)
{
	DOM_SID tmp_sid;
	uint32 rid;
	int type;

	sid_copy(&tmp_sid, sid);
	sid_split_rid(&tmp_sid, &rid);
	if (!sid_equal(&global_sam_sid, &tmp_sid))
	{
		return False;
	}

	if((id == NULL) || (rid < 1000))
		return False;

	type = uxrids_rid_posix_type(rid);
	id->id = uxrids_rid_posix_id(rid);

	switch (type)
	{
		case RID_TYPE_USER:
		{
			id->type = SURS_POSIX_UID;
			return True;
		}
		case RID_TYPE_ALIAS:
		case RID_TYPE_GROUP:
		{
			id->type = SURS_POSIX_GID;
			return True;
		}
		default:
		{
			break;
		}
	}
	return False;
}

/******************************************************************
 converts UNIX id + type to a SID.  the Domain SID is,
 and can only be, our own SID.
 ********************************************************************/
BOOL surs_algdomonly_unixid_to_sam_sid(const SURS_POSIX_ID *id, DOM_SID *sid,
				BOOL create)
{
	sid_copy(sid, &global_sam_sid);
	switch (id->type)
	{
		case SURS_POSIX_UID:
		{
			sid_append_rid(sid, uxrids_uid_to_user_rid(id->id));
			return True;
		}
		case SURS_POSIX_GID:
		{
			if (lp_server_role() == ROLE_DOMAIN_MEMBER ||
			    lp_server_role() == ROLE_STANDALONE)
			{
				sid_append_rid(sid,
					uxrids_gid_to_alias_rid(id->id));
			}
			else
			{
				sid_append_rid(sid,
					uxrids_gid_to_group_rid(id->id));
			}
			return True;
		}
	}
	return False;
}

