/* 
   Unix SMB/Netbios implementation.
   Version 2.1.
   Copyright (C) Luke Kenneth Casson Leighton 1996-1999,
   Copyright (C) Andrew Tridgell              1994-1999,
   Copyright (C) Jean Francois Micouleau      1998-1999.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "includes.h"
#include "libsamba.h"
#include "rpc_client.h"
#include "rpcclient.h"
#include "nterr.h"

extern FILE* out_hnd;

/****************************************************************************
****************************************************************************/
NTSTATUS cmd_eventlog(struct client_info *info, int argc, char *argv[])
{
	NTSTATUS ret = NT_STATUS_OK;
	POLICY_HND hnd;
	uint32 flags = EVENTLOG_READ_SEQUENTIAL|EVENTLOG_READ_BACKWARD;
	uint32 number;
	uint32 offset;
	EVENTLOGRECORD ev;

	char *journal = NULL;

	fstring srv_name;
	fstrcpy(srv_name, "\\\\");
	fstrcat(srv_name, info->dest_host);
	strupper(srv_name);

	if (argc > 1)
	{
		journal = argv[1];
	}

	if (!event_open(srv_name, journal, &hnd))
		return NT_STATUS_UNSUCCESSFUL;

	if (!event_numofeventlogrec(&hnd, &number))
	{
		event_close(&hnd);
		return NT_STATUS_UNSUCCESSFUL;
	}

	fprintf(out_hnd, "Number of events: %d\n", number);

	display_eventlog_eventrecord(out_hnd, ACTION_HEADER, &ev);

	for (offset = 0; offset < number; offset++)
	{
		uint32 num_of_bytes=0;
	
		/* try once with a empty buffer, then with the correct size */
		if (!event_readeventlog(&hnd, number, flags, offset,
					&num_of_bytes, &ev) ||
		    !event_readeventlog(&hnd, number, flags, offset,
					&num_of_bytes, &ev)) {
			ret = NT_STATUS_UNSUCCESSFUL;
			break;
		}

		display_eventlog_eventrecord(out_hnd, ACTION_ENUMERATE, &ev);
	}

	display_eventlog_eventrecord(out_hnd, ACTION_FOOTER, &ev);

	event_close(&hnd);

	return ret;
}
